use std::str::FromStr;

use color_eyre::eyre;
use eyre::eyre;

use crate::lib::{ParseLines, IterExt};


#[derive(Debug)]
enum Segment {
    A = 1,
    B = 2,
    C = 4,
    D = 8,
    E = 16,
    F = 32,
    G = 64, // wheee!
}
use Segment::*;

impl Segment {
    fn from_char(c: char) -> eyre::Result<Self> {
        match c {
            'a' => Ok(A),
            'b' => Ok(B),
            'c' => Ok(C),
            'd' => Ok(D),
            'e' => Ok(E),
            'f' => Ok(F),
            'g' => Ok(G),
            _ => Err(eyre!("Invalid character for segment: {}", c)),
        }
    }
}


#[derive(Default, Debug)]
struct SegmentSet {
    data: u8,
    len: u8,
}

impl SegmentSet {
    fn contains(&self, segment: Segment) -> bool {
        self.data & (segment as u8) > 0
    }

    fn insert(&mut self, segment: Segment) {
        self.data |= segment as u8;
        self.len += 1;
    }

    fn remove(&mut self, segment: Segment) {
        self.data ^= segment as u8;
        self.len -= 1
    }

    fn len(&self) -> u8 {
        self.len
    }

    fn union(&self, other: &Self) -> Self {
        self.data | other.data
    }

    fn intersect(&self, other: &Self) -> Self {
        self.data & other.data
    }
}

impl FromStr for SegmentSet {
    type Err = eyre::Report;
    fn from_str(s: &str) -> eyre::Result<Self> {
        let mut set = Self::default();
        for c in s.chars() {
            set.insert(Segment::from_char(c)?);
        }
        Ok(set)
    }
}


#[derive(Default, Debug)]
struct Screen {
    patterns: [SegmentSet; 10],
    output: [SegmentSet; 4],
}

impl FromStr for Screen {
    type Err = eyre::Report;
    fn from_str(s: &str) -> eyre::Result<Self> {
        let mut screen = Screen::default();
        let (patterns, output) = s.split(" | ").take_pair()?;
        
        for (i, patt) in patterns.split(' ').enumerate() {
            screen.patterns[i] = patt.parse::<SegmentSet>()?;
        }
        for (i, out) in output.split(' ').enumerate() {
            screen.output[i] = out.parse::<SegmentSet>()?;
        }
        Ok(screen)
    }
}


const DIGIT_SEGMENTS = {
    let segment_strs = [
        "abcefg", "cf", "acdeg", "acdfg",
        "bcdf", "abdfg", "abdefg", "acf",
        "abcdefg", "abcdfg",
    ]

    let sets = [SegmentSet::default(), 10]
    for (i, s) in segment_strs.iter().enumerate() {
        sets[i] = s.parse::<SegmentSet>().unwrap()
    }
    sets
}


fn part1(screens: &[Screen]) -> usize {
    screens.iter()
        .flat_map(|d| d.output.iter())
        .filter(|s| matches!(s.len(), 2|3|4|7))
        .count()
}


pub fn run(data: &str) -> eyre::Result<(usize, usize)> {
    let screens = data
        .parse_lines::<Screen>()
        .collect::<Result<Vec<_>, _>>()?;

    Ok((part1(&screens), 0))
}
